/*-------------------------------------------------------------------
 *
 *  @file util.c
 *
 *  @author Collin R. Mulliner <collin@mulliner.org>
 *
 *  This is part of the Bluetooth Location Tracker (BLT) project.
 *
 *-------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include "util.h"


char* scanWithEsc(char *in, char *out, int outleng)
{
	int c = 0;
	int esc = 0;
	
	
	if (in != NULL) {
		while (*in != 0 && c < outleng && *in != '\n' && *in != '\r') {
			if (*in == '\\') {
				if (esc == 0) {
					esc = 1;
					in++;
					continue;
				}
			}
			if (*in == '"' && esc == 0) break;
			out[c] = *in;
			in++;
			c++;
			esc = 0;
		}
	}
	return(in);
}

void printWithEsc(char *in, char *out, int outleng)
{
	int c = 0;
	

	if (in != NULL && out != NULL) {
		while (*in != 0 && c < outleng && *in != '\n' && *in != '\r') {
			if (*in == '"' || *in == '\\') {
				out[c] = '\\';
				c++;
			}
			out[c] = *in;
			in++;
			c++;
		}
	}
}

/**
 *  @brief copy one string to a other
 *
 *  @param in        src string
 *  @param out       dst string
 *  @param delimiter character in src where copy should stop
 *  @param outleng   size of dst
 *
 *  @returns nothing
 */
void strcpyendchr(char *in, char *out, char delimiter, int outleng)
{
	int c = 0;


	if (in != NULL) {       
		while (*in != 0 && *in != delimiter && c < outleng && *in != '\n' && *in != '\r') {
			out[c] = *in;
			in++;
			c++;
		}
	}
	out[c] = 0;
}

/**
 *  @brief parse a colon separated MAC/BD_ADDR and return the 6 bytes
 *
 *  @param in  input (e.g. AA:BB:CC:DD:EE:FF)
 *  @param out output
 *
 *  @returns nothing
 */
void parsemac(char *in, char *out)
{
	unsigned char i, i2, t;
	char *p;
	
	
	i2 = 2;
	p = in;
	for (i = 0; i < 6; i++) {
		t = in[i2];
		in[i2] = 0;
		out[i] = (unsigned char) strtoul(p, NULL, 16);
		in[i2] = t;
		p = in + i2 + 1;
		i2 += 3;
	}
}

/**
 *  @brief write n bytes to socket
 *
 *  this is taken from: unix network programming by W.R. Stevens
 *  
 *  @param fd   file descriptor
 *  @param vptr bytes to write out
 *  @param n    number of bytes to write
 *
 *  @returns -1 on failure "n" on success
 */
ssize_t nwrite(int fd, const void *vptr, size_t n)
{
	size_t nleft;
	ssize_t nwritten;
	const char *ptr;
	
	
	ptr = vptr;
	nleft = n;
	while (nleft > 0) {
		if ((nwritten = write(fd, ptr, nleft)) <= 0) {
			if (errno == EINTR) {
				nwritten = 0;
			}
			else {
				return(-1);
			}
		}
		nleft -= nwritten;
		ptr += nwritten;
	}
	return(n);
}

/**
 *  @brief checks if data is readable from the fd
 *
 *  @param fd   file descriptor to check
 *  @param wait number of seconds to wait for bytes
 *
 *  @returns >=1 when data is available else 0 is returned
 */
int fd_canread(int fd, int wait)
{
	fd_set rfds;
	struct timeval tv;


	FD_ZERO(&rfds);
	FD_SET(fd, &rfds);
	tv.tv_sec = wait;
	tv.tv_usec = 0;

	return(select(fd+1, &rfds, NULL, NULL, &tv));
}

/**
 *  @brief get line from buffer
 *
 *  This is only for internal use by readbuf!
 *
 *  @param buf    pointer to readbuf_t struct
 *  @param out    output buffer
 *  @param maxout size of output buffer
 *
 *  @returns 1 if line could be read, 0 if not
 */
int readbufGetLine(struct readbuf_t *buf, char *out, int maxout)
{
	char *in2 = buf->buffer;
	char *in = buf->buffer;
	int op = 0;
	

	while (*in2 != 0 && *in2 != '\n' && op < maxout) {
		out[op] = *in2;
		op++;
		in2++;
	}
	/* we only accept \n as line end */
	if (*in2 != '\n') {
		out[0] = 0;
		return(0);
	}
	else {
		out[op] = 0;
		op = 0;
		while (*in2 == '\n' || *in2 == '\r') in2++;
		while (*in2 != 0) {
			*in = *in2;
			in++;
			in2++;
			op++;
		}
		buf->leng = op;
		*in = 0;
		return(1);
	}
}

/**
 *  @brief read line from file descriptor
 *
 *  @param buf    pointer to readbuf struct
 *  @param fd     file descriptor to read from
 *  @param out    output buffer
 *  @param maxout size ouf output buffer
 *  @param wait   seconds to wait for input
 *
 *  @returns  1  if line is read
 *            0  if timeout
 *           -1  on error
 */
int readbuf(struct readbuf_t *buf, int fd, char *out, int maxout, int wait)
{
	int result = 0;
	

	do {
		if (readbufGetLine(buf, out, maxout)) {
			return(1);
		}
		else {
			if (fd_canread(fd, wait)) {
				reread:
				if ((result = read(fd, &buf->buffer[buf->leng], (buf->size - buf->leng)-1)) < 0) {
					if (result == EINTR) {
						result = 0;
						goto reread;
					}
					else {
						return(-1);
					}
				}
				else if (result == 0) {
					return(-1);
				}
				else {
					buf->leng += result;
					buf->buffer[buf->leng] = 0;
					result = 0;
				}
			}
			else {
				return(0);
			}
		}
	} while(1);
}

/**
 *  @brief init readbuf descriptor
 *
 *  @param buf  pointer to descriptor
 *  @param size buffer size
 *
 *  @returns nothing
 */
void readbufInit(struct readbuf_t *buf, int size)
{
	buf->buffer = malloc(size);
	buf->size = size;
	buf->leng = 0;
}

/**
 *  @brief destroy readbuf descriptor
 *
 *  @param buf pointer to descriptor
 *
 *  @returns nothing
 */
void readbufDestroy(struct readbuf_t *buf)
{
	if (buf->buffer != NULL) free(buf->buffer);
	buf->buffer = NULL;
	buf->size = 0;
	buf->leng = 0;
}
